package test.client;

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.rmi.RemoteException;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.rpc.ServiceException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.sax.SAXResult;
import javax.xml.transform.stax.StAXSource;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.impl.builder.SAXOMBuilder;
import org.apache.axis2.AxisFault;
import org.exolab.castor.xml.MarshalException;
import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.UnmarshalHandler;
import org.exolab.castor.xml.Unmarshaller;
import org.exolab.castor.xml.ValidationException;
import org.gjt.xpp.XmlPullParser;
import org.gjt.xpp.XmlPullParserException;
import org.gjt.xpp.XmlPullParserFactory;
import org.gjt.xpp.XmlStartTag;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;


import com.castor.client.JokerStub;
import com.wsimport.Joker;
import com.wsimport.Joker_Service;
import com.wsimport.MethodRequest;
import com.wsimport.MethodResponse;


public class Client {
	
	private String testString 								= null;
    private static final int BUFFER_SIZE					= 10000;  
    private XmlPullParserFactory factory 					= null;
    private XmlPullParser pp 								= null;
	private JAXBContext _location_context 					= null;
	private Joker wsImportJoker 									= null;
	private com.axis.client.wsdl.Joker_PortType axisJoker 	= null;
	private JokerStub stub 									= null;
	private SAXOMBuilder builder 							= null;
	private Unmarshaller unmarshaller 						= null;
	private UnmarshalHandler unmarshalHandler 				= null;
	private com.castor.client.data.MethodResponse methodResponse 	= null;
	private com.castor.client.data.MethodRequest methodRequest 		= null;
	private ContentHandler contentHandler 							= null;
	
	public Client() {
		super();
    	testString = "Республика островов Фиджи, расположенная в южной части Тихого океана, против того, чтобы следующая версия Microsoft Windows называлась ее именем.";
    	try {
			testString = java.net.URLEncoder.encode(testString,"UTF-8");
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		wsImportJoker =  new Joker_Service().getJokerPort();
		try {
			axisJoker = new com.axis.client.wsdl.Joker_ServiceLocator().getJokerPort();
		} catch (ServiceException e) {
			e.printStackTrace();
		}
    	try {
			factory = XmlPullParserFactory.newInstance();
			pp = factory.newPullParser();
		} catch (XmlPullParserException e) {
			e.printStackTrace();
		}
		try {
			stub = new JokerStub();
		} catch (AxisFault e) {
			e.printStackTrace();
		}
    	stub._getServiceClient().getOptions().setAction("shift");
    	builder = new SAXOMBuilder();
        unmarshaller = new Unmarshaller(com.castor.client.data.MethodResponse.class);
        unmarshalHandler = unmarshaller.createHandler();  
        methodResponse = new com.castor.client.data.MethodResponse();
        methodRequest = new com.castor.client.data.MethodRequest();
        try {
			contentHandler = Unmarshaller.getContentHandler(unmarshalHandler);
		} catch (SAXException e) {
			e.printStackTrace();
		}
	}

    public static void main(String[] args) throws Exception {

    	long startTime;
    	
    	Client c = new Client();

        System.out.println("\nPlainTest Test");
    	startTime = System.currentTimeMillis();
    	System.out.println("Result: " + c.xmlPullParserTest());        
        System.out.println("Time elapsed: "+(System.currentTimeMillis()-startTime)+"ms");    	
    	
        System.out.println("\nWSImport Test");
    	startTime = System.currentTimeMillis();
    	System.out.println("Result: " + c.wSImportTest());        
        System.out.println("Time elapsed: "+(System.currentTimeMillis()-startTime)+"ms");
        
        System.out.println("\nAxisTest Test");
    	startTime = System.currentTimeMillis();
    	System.out.println("Result: " + c.axisTest());        
        System.out.println("Time elapsed: "+(System.currentTimeMillis()-startTime)+"ms");        
        
        System.out.println("\nCastor Test");
    	startTime = System.currentTimeMillis();
    	System.out.println("Result: " + c.castorTest());        
        System.out.println("Time elapsed: "+(System.currentTimeMillis()-startTime)+"ms");        
        
    }
    
    private String castorTest() throws MarshalException, ValidationException, IOException{
    	methodRequest.setReqString(testString);
        Marshaller.marshal(methodRequest, builder);
        OMElement response = stub.shift(builder.getRootElement());
        StAXSource staxSource = new StAXSource(response.getXMLStreamReader());
        try {
            TransformerFactory.newInstance().newTransformer().transform(staxSource, new SAXResult(contentHandler));
            methodResponse = (com.castor.client.data.MethodResponse) unmarshalHandler.getObject();
        } catch (TransformerException e) {
        	e.printStackTrace();
        }      
        return java.net.URLDecoder.decode(methodResponse.getResString(),"UTF-8");
    }
    
    private String axisTest() throws RemoteException, UnsupportedEncodingException{
    	com.axis.client.wsdl.MethodRequest axisReq = new com.axis.client.wsdl.MethodRequest();
    	axisReq.setReqString(testString);
    	System.out.println("Request: "+printObject(axisReq));
    	com.axis.client.wsdl.MethodResponse  response = axisJoker.shift(axisReq);
    	return java.net.URLDecoder.decode(response.getResString(),"UTF-8");
    }
    
    private String wSImportTest() throws UnsupportedEncodingException{
    	MethodRequest request = new MethodRequest();
    	request.setReqString(testString);
    	System.out.println("Request: "+printObject(request));
    	MethodResponse response = wsImportJoker.shift(request);
    	return java.net.URLDecoder.decode(response.getResString(),"UTF-8");
    }
    
    private String xmlPullParserTest() throws XmlPullParserException, IOException{
        String request = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" +
        "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\" " +
        "xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" " +
        "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"><soapenv:Body>" +
        "<methodRequest reqString=\""+testString+"\" xmlns=\"http://localhost\"/>" +
        "</soapenv:Body></soapenv:Envelope>";
    	String response = getResponse("http://127.0.0.1:80/joker/wsbase",request,
    			"http://127.0.0.1:80/joker/wsbase/shift",10000);
    	String ret = "";
    	pp.setInput(new StringReader(response));
    	byte type;  // received event type
    	XmlStartTag stag = factory.newStartTag();
	  	while(true) {
	  		type = pp.next();
		  	if(type == XmlPullParser.START_TAG){
		  		pp.readStartTag(stag);
		  		if("methodResponse".equals(stag.getLocalName())){
		  			ret = stag.getAttributeValueFromRawName("resString");
		  			break;
		  		}
		  	}
		  	if(type == XmlPullParser.END_DOCUMENT) break;
	  	}
    	return java.net.URLDecoder.decode(ret,"UTF-8");    	
    }
    
    private String getResponse(String urlString, String requestXML, String action, int timeOut) {
    	URL targetURL = null;
		try {
			targetURL = new URL(urlString);
		} catch (MalformedURLException e) {
			e.printStackTrace();
		}
    	URLConnection con = null;
		try {
			con = targetURL.openConnection();
		} catch (IOException e) {
			e.printStackTrace();
		}
    	con.setDoInput (true);
    	con.setDoOutput (true);
    	con.setRequestProperty("Content-Type", "text/xml");
    	con.setRequestProperty("Content-Length",String.valueOf(requestXML.getBytes().length));
    	con.setRequestProperty("charset", "utf-8");	    
    	con.setRequestProperty("SOAPAction", "\"" + action + "\""); 
        if (timeOut > 0) con.setReadTimeout(timeOut);
        con.setUseCaches(false);
        con.setAllowUserInteraction(false);

		// send data:
		DataOutputStream dataOut = null;
		try {
			dataOut = new DataOutputStream(con.getOutputStream());
		} catch (IOException e) {
			e.printStackTrace();
		}
		try {
			dataOut.writeBytes(requestXML);
			dataOut.flush();
			dataOut.close();
		} catch (IOException e) {
			e.printStackTrace();
		}

		// read response:
        InputStream is = null;
        StringBuffer out = null;
		try {
			is = con.getInputStream();
			out = new StringBuffer();
			byte[] b = new byte[BUFFER_SIZE];
			for (int n; (n = is.read(b)) != -1;) {
			    out.append(new String(b, 0, n));
			}
			is.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
       
		return out.toString();
    } 
    
    private JAXBContext getJAXBLocationContext() throws JAXBException{
    	return (_location_context == null) ? JAXBContext.newInstance(MethodResponse.class.getPackage().getName()) : _location_context;
    }
    
	private String printObject(Object o){
		JAXBContext c;
		StringWriter sw = new StringWriter();
		try {
			c = getJAXBLocationContext();
			(c.createMarshaller()).marshal(o, sw);
		} catch (JAXBException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return sw.toString();
	}	    
}
